"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Recurrence_1 = require("../ComplexProperties/Recurrence/Patterns/Recurrence");
var NoEndRecurrenceRange_1 = require("../ComplexProperties/Recurrence/Ranges/NoEndRecurrenceRange");
var EndDateRecurrenceRange_1 = require("../ComplexProperties/Recurrence/Ranges/EndDateRecurrenceRange");
var NumberedRecurrenceRange_1 = require("../ComplexProperties/Recurrence/Ranges/NumberedRecurrenceRange");
var XmlElementNames_1 = require("../Core/XmlElementNames");
var ServiceXmlDeserializationException_1 = require("../Exceptions/ServiceXmlDeserializationException");
var ExtensionMethods_1 = require("../ExtensionMethods");
var Strings_1 = require("../Strings");
var PropertyDefinition_1 = require("./PropertyDefinition");
/**
 * @internal Represenrs recurrence property definition.
 */
var RecurrencePropertyDefinition = (function (_super) {
    __extends(RecurrencePropertyDefinition, _super);
    /**
     * @internal Initializes a new instance of the **RecurrencePropertyDefinition** class.
     *
     * @param   {string}                    propertyName     Name of the property (added to workaround reflection based initialization of Names).
     * @param   {string}                    xmlElementName   Name of the XML element.
     * @param   {string}                    uri              The URI.
     * @param   {PropertyDefinitionFlags}   flags            The flags.
     * @param   {ExchangeVersion}           version          The version.
     */
    function RecurrencePropertyDefinition(propertyName, xmlElementName, uri, flags, version) {
        _super.call(this, propertyName, xmlElementName, uri, flags, version);
    }
    /**
     * Gets the recurrence from string.
     *
     * @param   {string}   recurranceString   The recurrance string.
     * @return  {Recurrence}    Recurrence
     */
    RecurrencePropertyDefinition.GetRecurrenceFromString = function (recurranceString) {
        var recurrence;
        switch (recurranceString) {
            case XmlElementNames_1.XmlElementNames.RelativeYearlyRecurrence:
                recurrence = new Recurrence_1.Recurrence.RelativeYearlyPattern();
                break;
            case XmlElementNames_1.XmlElementNames.AbsoluteYearlyRecurrence:
                recurrence = new Recurrence_1.Recurrence.YearlyPattern();
                break;
            case XmlElementNames_1.XmlElementNames.RelativeMonthlyRecurrence:
                recurrence = new Recurrence_1.Recurrence.RelativeMonthlyPattern();
                break;
            case XmlElementNames_1.XmlElementNames.AbsoluteMonthlyRecurrence:
                recurrence = new Recurrence_1.Recurrence.MonthlyPattern();
                break;
            case XmlElementNames_1.XmlElementNames.DailyRecurrence:
                recurrence = new Recurrence_1.Recurrence.DailyPattern();
                break;
            case XmlElementNames_1.XmlElementNames.DailyRegeneration:
                recurrence = new Recurrence_1.Recurrence.DailyRegenerationPattern();
                break;
            case XmlElementNames_1.XmlElementNames.WeeklyRecurrence:
                recurrence = new Recurrence_1.Recurrence.WeeklyPattern();
                break;
            case XmlElementNames_1.XmlElementNames.WeeklyRegeneration:
                recurrence = new Recurrence_1.Recurrence.WeeklyRegenerationPattern();
                break;
            case XmlElementNames_1.XmlElementNames.MonthlyRegeneration:
                recurrence = new Recurrence_1.Recurrence.MonthlyRegenerationPattern();
                break;
            case XmlElementNames_1.XmlElementNames.YearlyRegeneration:
                recurrence = new Recurrence_1.Recurrence.YearlyRegenerationPattern();
                break;
            default:
                //info: //ref: cannot throw, need to keep null/undefined to successfully parse XMLJsObject
                //throw new ServiceXmlDeserializationException(StringHelper.Format(Strings.InvalidRecurrencePattern, recurranceString));
                break;
        }
        return recurrence;
    };
    /**
     * Gets the recurrence range.
     *
     * @param   {string}            recurrenceRangeString   The recurrence range string.
     * @return  {RecurrenceRange}   RecurrenceRange
     */
    RecurrencePropertyDefinition.GetRecurrenceRange = function (recurrenceRangeString) {
        var range;
        switch (recurrenceRangeString) {
            case XmlElementNames_1.XmlElementNames.NoEndRecurrence:
                range = new NoEndRecurrenceRange_1.NoEndRecurrenceRange();
                break;
            case XmlElementNames_1.XmlElementNames.EndDateRecurrence:
                range = new EndDateRecurrenceRange_1.EndDateRecurrenceRange();
                break;
            case XmlElementNames_1.XmlElementNames.NumberedRecurrence:
                range = new NumberedRecurrenceRange_1.NumberedRecurrenceRange();
                break;
            default:
                //info: //ref: cannot throw, need to keep null/undefined to successfully parse XMLJsObject
                //throw new ServiceXmlDeserializationException(StringHelper.Format(Strings.InvalidRecurrenceRange, recurrenceRangeString)); 
                break;
        }
        return range;
    };
    /**
     * @internal Loads the property value from json.
     *
     * @param   {any}               jsObject         The JSON value.  Can be a JsonObject, string, number, bool, array, or null.
     * @param   {ExchangeService}   service       The service.
     * @param   {PropertyBag}       propertyBag   The property bag.
     */
    RecurrencePropertyDefinition.prototype.LoadPropertyValueFromXmlJsObject = function (jsObject, service, propertyBag) {
        var recurrence;
        var range;
        var keys = "";
        for (var key in jsObject) {
            if (key.indexOf("__") === 0)
                continue;
            if (jsObject.hasOwnProperty(key)) {
                recurrence = RecurrencePropertyDefinition.GetRecurrenceFromString(key) || recurrence;
                range = RecurrencePropertyDefinition.GetRecurrenceRange(key) || range;
                keys += key + ",";
            }
        }
        if (keys.length > 1 && keys.lastIndexOf(",") === keys.length - 1) {
            keys = keys.substr(0, keys.length - 1);
        }
        if (!recurrence) {
            throw new ServiceXmlDeserializationException_1.ServiceXmlDeserializationException(ExtensionMethods_1.StringHelper.Format(Strings_1.Strings.InvalidRecurrencePattern, keys));
        }
        if (!range) {
            throw new ServiceXmlDeserializationException_1.ServiceXmlDeserializationException(ExtensionMethods_1.StringHelper.Format(Strings_1.Strings.InvalidRecurrenceRange, keys));
        }
    };
    /**
     * @internal Writes the property value to XML.
     *
     * @param   {EwsServiceXmlWriter}   writer              The writer.
     * @param   {PropertyBag}           propertyBag         The property bag.
     * @param   {boolean}               isUpdateOperation   Indicates whether the context is an update operation.
     */
    RecurrencePropertyDefinition.prototype.WritePropertyValueToXml = function (writer, propertyBag, isUpdateOperation) {
        var value = propertyBag._getItem(this);
        if (value) {
            value.WriteToXml(writer, XmlElementNames_1.XmlElementNames.Recurrence);
        }
    };
    return RecurrencePropertyDefinition;
}(PropertyDefinition_1.PropertyDefinition));
exports.RecurrencePropertyDefinition = RecurrencePropertyDefinition;
